'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _memory = require('./storage/memory');

var _memory2 = _interopRequireDefault(_memory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const debug = (0, _debug2.default)('talktalk:dispather');

class Dispatcher {

  constructor(sender, store = new _memory2.default()) {
    this.handlers = [];
    this.handlersMap = {};

    this.store = store;
    this.sender = sender;
  }

  buildPostback(_Handler, context, sender) {
    return { type: 'postback', context, target: _Handler.name, sender };
  }

  handleMessages(messages) {
    debug('Handling messages', messages);
    return Promise.all(messages.map(message => this.handleMessage(message)));
  }

  handleMessage(message) {
    var _this = this;

    return _asyncToGenerator(function* () {
      switch (message.type) {
        case 'postback':
          return _this._handlePostback(message);
        case 'message':
          return _this._handleMessage(message);
        default:
          throw new Error('Message type not supported');
      }
    })();
  }

  _saveOrClearSession(userId, session) {
    if (!session.context) {
      debug('No context. Clearing session', session);
      return this.store.clearEntry(userId);
    }
    debug('Saving session');
    return this.store.setEntry(userId, session);
  }

  _fetchSession(userId) {
    return this.store.getEntry(userId);
  }

  _handleMessage(message) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      debug('Handling message');
      const session = yield _this2._fetchSession(message.sender);
      debug('... message', message);
      if (!session) {
        debug('Found no session');
        return _this2._handleFirstMessage(message, _this2.handlers.slice(0));
      }
      debug('Found session', session);
      return _this2._handleSessionMessage(message, session);
    })();
  }

  _handlePostback(postback) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const _Handler = _this3.handlersMap[postback.target];
      if (!_Handler) {
        return;
      }
      const handler = new _Handler(function (reply) {
        return _this3.sender(reply, postback);
      }, postback.sender);
      debug('Handling postback with handler id', postback.target, 'and payload', postback.context);
      const context = yield handler.handleJump(postback.context);
      return _this3._messagePostProcess(handler, context, postback);
    })();
  }

  _handleFirstMessage(message, handlers) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      if (!handlers.length) {
        debug('Found no handler...');
        throw new Error('No handler found');
      }
      const _Handler = handlers[0];
      debug('Trying', _Handler.name);
      const handler = new _Handler(function (reply) {
        return _this4.sender(reply, message);
      }, message.sender);
      const context = yield handler.handleMessage(message);
      debug('Finished message, got context', context);
      return _this4._messagePostProcess(handler, context, message, handlers.slice(1));
    })();
  }

  _handleSessionMessage(message, { handler: handlerId, context }) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      const _Handler = _this5.handlersMap[handlerId];
      const handler = new _Handler(function (reply) {
        return _this5.sender(reply, message);
      }, message.sender);
      debug('Trying', handlerId);
      const newContext = yield handler.handleMessage(message, context);
      debug('Finished handling message', newContext);
      return _this5._messagePostProcess(handler, newContext, message, _this5.handlers.slice(0));
    })();
  }

  _messagePostProcess(handler, context, message, handlers = []) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      const jumber = handler.jumper;
      if (jumber) {
        debug('Jumping...');
        const { handler: newHandler, context: newContext } = yield jumber();
        debug('... to ', newHandler.constructor.name, 'with context', newContext);
        return _this6._messagePostProcess(newHandler, newContext, message, _this6.handlers);
      }
      if (handler.hasSentReply) {
        yield _this6._saveOrClearSession(message.sender, { handler: handler.constructor.name, context });
        return;
      }
      if (message.type === 'postback') {
        return;
      }
      debug('Handler should not send slicing ');
      return _this6._handleFirstMessage(message, handlers);
    })();
  }

  registerHandler(Handler) {
    this.handlers.push(Handler);
    this.handlersMap[Handler.name] = Handler;
  }
}
exports.default = Dispatcher;